"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlindsCmdPlatform = void 0;
const settings_1 = require("./settings");
const blindsCmd_blind_1 = require("./blindsCmd-blind");
const util_1 = __importDefault(require("util"));
class BlindsCmdPlatform {
    constructor(log, config, api) {
        this.accessories = [];
        this.blinds = [];
        this.debugMode = false;
        this.api = api;
        this.log = log;
        // No configuration found - we won't start.
        if (!config) {
            return;
        }
        // If we don't have any blinds configured, we're done here.
        if (!config.blinds) {
            this.log.error("No blinds have been configured.");
            return;
        }
        // Force this to BlindConfig.
        this.config = {
            blinds: config.blinds
        };
        // Capture configuration parameters.
        if (config.debug) {
            this.debugMode = config.debug === true;
            this.debug("Debug logging on. Expect a lot of data.");
        }
        // Avoid a prospective race condition by waiting to configure our blinds until Homebridge is done
        // loading all the cached accessories it knows about, and calling configureAccessory() on each.
        api.on("didFinishLaunching" /* DID_FINISH_LAUNCHING */, this.configureBlinds.bind(this));
    }
    // This gets called when homebridge restores cached accessories at startup.
    configureAccessory(accessory) {
        // Add this to the accessory array so we can track it.
        this.accessories.push(accessory);
    }
    // Configure our blinds.
    configureBlinds() {
        // Loop through each configured blind and instantiate it.
        for (const blindsConfig of this.config.blinds) {
            // No name or up or down commands defined, we're done.
            if (!blindsConfig.up || !blindsConfig.down || !blindsConfig.name) {
                this.log("Name, up, and down commands are required configuration parameters.");
                continue;
            }
            this.blinds.push(new blindsCmd_blind_1.Blind(this, blindsConfig));
        }
        // Remove any blinds that are no longer configured.
        for (const accessory of this.accessories) {
            // We've configured this blind already, let's keep going.
            if (this.blinds.some(x => x.accessory.UUID === accessory.UUID)) {
                continue;
            }
            // If we have a blind that we don't have a configuration for, remove it and inform the user.
            this.log.info("Removing blind from HomeKit: %s.", accessory.displayName);
            this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
        }
    }
    // Utility for debug logging.
    debug(message, ...parameters) {
        if (this.debugMode) {
            this.log(util_1.default.format(message, ...parameters));
        }
    }
}
exports.BlindsCmdPlatform = BlindsCmdPlatform;
//# sourceMappingURL=blindsCmd-platform.js.map